#include "menge.h"

//Standardkonstruktor
menge::menge()
{
    size = 1;
    anzahl = 0;
    try{
        feld = new int;
    } catch(bad_alloc xa){
        cerr << "Fehler beim Erzeugen!\n";
    }
}


//Konstruktor
menge::menge(int size)
{
    this->size = size;
    anzahl = 0;
    try{
        feld = new int[size];
    } catch(bad_alloc xa){
        cerr << "Fehler beim Erzeugen!\n";
    }
}


//Copy-Konstruktor
menge::menge(const menge& m)
{
    anzahl = m.anzahl;
    size = m.size;
    
    try{
        feld = new int[size];
    } catch(bad_alloc xa) {
        cerr << "Fehler im Copy-Konstruktor\n";
    }
    for(int i=0; i<anzahl; i++)
        feld[i] = m.feld[i];
}


//Zuweisung (m1 = m2)
//return m1
menge& menge::operator=(const menge& m)
{
    //wenn m2 grer
    if(size < m.size)
        resize(m.size);
    
    anzahl = m.anzahl;
    
    for(int i=0; i<anzahl; i++)
        feld[i] = m.feld[i];
    
    // * vor (zeiger auf objekt) <==> wert von objekt
    return *this;
}



void menge::resize(int size)
{
    if(this->size < size)
    {
        //werte sichern
        int *temp;
        temp = new int[anzahl];
        for(int i=0; i<anzahl; i++)
            temp[i] = feld[i];
        
        //feld vergrern und wiederherstellen
        feld = new int[size];
        for(int i=0; i<anzahl; i++)
            feld[i] = temp[i];
        
        //gre aktualisieren
        this->size = size;
        //speicher freigeben
        delete [] temp;
    }
}



//schauen, ob element in menge enthalten
int menge::IsElem(int element)
{
    for(int i=0; i<anzahl; i++)
        if(element == feld[i])
            return 1;
    return 0;
}



//Vereinigungsmenge (m1+m2)
//return temp = m1 + m2 = m1.operator+(m2);
menge menge::operator+(const menge& m)
{
    menge temp;
    //setze temp = m1
    temp = *this;
    
    for(int i=0; i<m.anzahl; i++)
    {
        //wenn temp zu klein, um noch elemente aufzunehmen
        if(temp.anzahl == temp.size)
            temp.resize(anzahl + 10);
        
        //wenn noch nicht enthalten, dann einfgen
        if(!temp.IsElem(m.feld[i]))
            temp.feld[temp.anzahl++] = m.feld[i];
    }
    return temp;
}


//Restmenge (m1-m2)
//return temp = m1 - m2 = m1.operator-(m2);
menge menge::operator-(const menge& m)
{
    menge temp;
    //setze temp = m1
    temp = *this;
    
    for(int i=0; i<temp.anzahl; i++)
        for(int k=0; k<m.anzahl; k++)
            if( temp.feld[i] == m.feld[k] )
                //letztes element an aktuelle position schieben
                //und anzahl erniedrigen
                temp.feld[i] = temp.feld[--temp.anzahl];
    return temp;
}



//Durchschnitt
//return temp = (m1 / m2)
menge menge::operator/(const menge& m)
{
    menge temp;
    
    //temp mu maximal soviele elemente aufnehmen knnen,
    //wie die grere menge elemente hat
    if(anzahl > m.anzahl)
        temp.resize(anzahl);
    else
        temp.resize(m.anzahl);
    
    
    for(int i=0; i<m.anzahl; i++)
        if(IsElem(m.feld[i]))
        //wenn element in m1 und m2, dann ab in temp
            temp.feld[temp.anzahl++] = m.feld[i];
    
    return temp;
}



//Teilmenge (m2 c m1)
int menge::IsPart(const menge& m)
{
    int not_part=0;
    
    for(int i=0; i<m.anzahl; i++)
        if( !IsElem(m.feld[i]) )
            not_part++;
    
    //wenn es elemente in m2 gibt, die nicht in m1 enthalten sind,
    //dann ist m2 keine teilemenge von m1
    if(not_part)
        return 0;
    return 1;
}



//Gleichheit
//  [(m1 c m2) && (m2 c m1)]  ==>  (m1 == m2)
int menge::operator==(menge m)
{
    // * vor (zeiger auf objekt) <==> wert von objekt
    //(m2 c m1)?  && (m1 c m2)?
    if( IsPart(m) && m.IsPart(*this) )
        return 1;
    return 0;
}




//Ungleichheit
//  [(m1 c m2) && (m2 c m1)]  ==>  (m1 != m2)
int menge::operator!=(menge m)
{
    if( IsPart(m) && m.IsPart(*this) )
        return 0;
    return 1;
}




menge operator+(const menge& m, int element)
{
    //schauen, ob element bereits in menge
    for(int i=0; i<m.anzahl; i++)
        if(element == m.feld[i])
            return m;   //wenn ja, dann m unverndert zurck
    
    //sonst :
    //um element aufnehmen zu knnen, mu temp um 1 grer sein als m elemente hat
    menge temp(m.anzahl + 1);
    for(int i=0; i<m.anzahl; i++)
        temp.feld[i] = m.feld[i];
    temp.feld[m.anzahl] = element;
    
    return temp;
}




menge operator+(int element, const menge& m)
{
    return m + element;
}



menge operator-(const menge& m, int element)
{
    menge temp = m;
    
    for(int i=0; i<temp.anzahl; i++)
        if(element == temp.feld[i])
            temp.feld[i] = temp.feld[--temp.anzahl];
    
    return temp;
}


//Ausgabe
ostream& operator<<(ostream& s, const menge& m)
{
    s << "M = { ";
    if(m.anzahl >= 1)
    {
        s << m.feld[0];
        for(int i=1; i<m.anzahl; i++)
             s << ", " << m.feld[i];
    }
    s << " }\n";
    return s;
}


//Eingabe
istream& operator>>(istream& i, menge& m)
{
    int element;
    
    cout << "Element eingeben: ";
    cin >> element;
    
    //schauen, ob element schon in menge enthalten
    if( !m.IsElem(element) )
    {
        //wenn nicht genug platz, um element aufzunehmn, vergrern
        if(m.anzahl == m.size)
            m.resize(m.anzahl+1);
        
        //element einfgen und dann anzahl erhhen
        m.feld[m.anzahl++] = element;
    }
    return i;
}

